@@ -22,6 +22,7 @@
 #include <stdint.h>
 #include <assert.h>
 #include <errno.h>
+#include <err.h>
 #include <linux/ioctl.h>
 #include <linux/usbdevice_fs.h>
 #include <linux/input.h>
@@ -165,11 +166,9 @@ ioctl_tree_insert(ioctl_tree * tree, ioctl_tree * node)
     }
 
     node->parent = node->type->insertion_parent(tree, node);
-    if (node->parent == NULL) {
-	fprintf(stderr, "ioctl_tree_insert: did not get insertion parent for node type %s ptr %p\n",
-		node->type->name, node);
-	abort();
-    }
+    if (node->parent == NULL)
+	errx(EXIT_FAILURE, "ioctl_tree_insert: did not get insertion parent for node type %s ptr %p",
+	     node->type->name, node);
 
     /* if the parent is the whole tree, then we put it as a sibling, not a
      * child */
@@ -357,6 +356,10 @@ ioctl_tree_execute(ioctl_tree * tree, ioctl_tree * last, IOCTL_REQUEST_TYPE id,
     ioctl_tree *i;
     int r, handled;
 
+    /* initialize return code */
+    assert(ret != NULL);
+    *ret = -1;
+
     DBG(DBG_IOCTL_TREE, "ioctl_tree_execute ioctl %X\n", (unsigned) id);
 
     t = ioctl_type_get_by_id(id);
@@ -369,8 +372,6 @@ ioctl_tree_execute(ioctl_tree * tree, ioctl_tree * last, IOCTL_REQUEST_TYPE id,
 	DBG(DBG_IOCTL_TREE, "  ioctl_tree_execute: stateless\n");
 	if (t->execute(NULL, id, arg, &r))
 	    *ret = r;
-	else
-	    *ret = -1;
 	return last;
     }
 
