@@ -22,6 +22,7 @@
 /* for O_TMPFILE */
 #define _GNU_SOURCE
 
+#include <err.h>
 #include <glib.h>
 #include <glib/gstdio.h>
 #include <string.h>
@@ -34,6 +35,7 @@
 #include <sys/sysmacros.h>
 #include <sys/un.h>
 #include <sys/vfs.h>
+#include <sys/xattr.h>
 #include <linux/usbdevice_fs.h>
 #include <linux/input.h>
 #include <linux/magic.h>
@@ -810,7 +812,7 @@ t_testbed_uevent_error(UMockdevTestbedFixture * fixture, UNUSED_DATA)
     fflush(stderr);
     rewind(stderr);
     g_assert(fgets(buf, sizeof(buf), stderr) != NULL);
-    g_assert_cmpstr(buf, ==, "ERROR: uevent_sender_send: No such device /devices/unknown\n");
+    g_assert_cmpstr(buf, ==, "ERROR: uevent_sender_send: Failed udev_device_new_from_syspath(/devices/unknown): Invalid argument\n");
     fclose(stderr);
     stderr = orig_stderr;
 #endif
@@ -1628,6 +1630,29 @@ t_testbed_add_from_string_dev_block(UMockdevTestbedFixture * fixture, UNUSED_DAT
     g_assert_cmpuint(stx.stx_uid, ==, getuid());
     g_assert(S_ISBLK(stx.stx_mode));
 #endif
+
+    /* listxattr() */
+    char xattr_buf[1024];
+    ssize_t res = listxattr("/dev/empty", xattr_buf, sizeof(xattr_buf));
+    g_assert_cmpint(res, >=, 0);
+
+    res = listxattr("/dev/sdf", xattr_buf, sizeof(xattr_buf));
+    g_assert_cmpint(res, >=, 0);
+
+    res = listxattr("/nonexisting", xattr_buf, sizeof(xattr_buf));
+    g_assert_cmpint(res, ==, -1);
+    g_assert_cmpint(errno, ==, ENOENT);
+
+    /* llistxattr() */
+    res = llistxattr("/dev/empty", xattr_buf, sizeof(xattr_buf));
+    g_assert_cmpint(res, >=, 0);
+
+    res = llistxattr("/dev/sdf", xattr_buf, sizeof(xattr_buf));
+    g_assert_cmpint(res, >=, 0);
+
+    res = listxattr("/nonexisting", xattr_buf, sizeof(xattr_buf));
+    g_assert_cmpint(res, ==, -1);
+    g_assert_cmpint(errno, ==, ENOENT);
 }
 
 static void
@@ -2044,10 +2069,8 @@ r 10 ^@response\n";
   g_assert_cmpint(fd, >=, 0);
   saddr.sun_family = AF_UNIX;
   snprintf(saddr.sun_path, sizeof(saddr.sun_path), "/dev/socket/chatter");
-  if (connect(fd, (struct sockaddr *) &saddr, sizeof(saddr)) < 0) {
-      perror("t_testbed_script_replay_socket_stream() connect");
-      abort();
-  }
+  if (connect(fd, (struct sockaddr *) &saddr, sizeof(saddr)) < 0)
+      err(EXIT_FAILURE, "t_testbed_script_replay_socket_stream() connect");
 
   /* should get initial greeting after 200 ms */
   ASSERT_EOF;
